"""
The models
"""

# Django
from django.db import models
from django.utils.translation import gettext_lazy as _


class TimezoneData(models.Model):
    """
    Available timezones imported from pytz
    """

    timezone_name = models.CharField(
        max_length=255, blank=False, unique=True, help_text=_("Name of the timezone")
    )

    utc_offset = models.CharField(
        max_length=255,
        blank=False,
        unique=False,
        help_text=_("UTC offset of the timezone"),
    )

    panel_id = models.CharField(
        max_length=255,
        blank=False,
        unique=True,
        help_text=_("ID of the timezone panel in frontend"),
    )

    def __str__(self) -> str:
        return str(self.timezone_name)

    class Meta:  # pylint: disable=too-few-public-methods
        """
        TimezoneData :: Meta
        """

        verbose_name = _("Timezone Data")
        verbose_name_plural = _("Timezone Data")
        default_permissions = ()


class Timezones(models.Model):
    """
    Configured timezones to display
    """

    panel_name = models.CharField(
        max_length=255,
        blank=False,
        unique=True,
        help_text=_("Name of the timezone panel"),
    )

    timezone = models.ForeignKey(
        TimezoneData,
        on_delete=models.CASCADE,
        blank=False,
        help_text=_("Selected timezone"),
    )

    is_enabled = models.BooleanField(
        default=True,
        help_text=_("Whether this timezone is enabled or not"),
    )

    def __str__(self) -> str:
        return str(self.panel_name)

    class Meta:  # pylint: disable=too-few-public-methods
        """
        TimezoneData :: Meta
        """

        verbose_name = _("Timezone")
        verbose_name_plural = _("Timezones")
        default_permissions = ()



"""
App settings and checks
"""

# Django
from django.apps import apps


def allianceauth_discordbot_active():
    """
    Check if allianceauth-discordbot is installed and active
    :return:
    """

    return apps.is_installed("aadiscordbot")



"""
Test checks for access to timezones
"""

# Django
from django.contrib.auth.models import Group
from django.test import TestCase
from django.urls import reverse

# AA Time Zones
from timezones.constants import AA_TIMEZONE_DEFAULT_PANELS
from timezones.models import TimezoneData, Timezones
from timezones.tests.utils import create_fake_user


class TestAccess(TestCase):
    """
    Test access to the views
    """

    @classmethod
    def setUpClass(cls) -> None:
        """
        Set up groups and users
        """

        super().setUpClass()

        cls.group = Group.objects.create(name="Superhero")

        cls.user_1002 = create_fake_user(
            character_id=1002, character_name="Bruce Wayne"
        )

    def test_default_timezones(self):
        """
        Test default timezones
        :return:
        """

        # given
        self.client.force_login(self.user_1002)

        # when
        res = self.client.get(reverse("timezones:index"))

        # then
        self.assertListEqual(res.context["timezones"], AA_TIMEZONE_DEFAULT_PANELS)

    def test_custom_timezones(self):
        """
        Test custom timezones
        :return:
        """

        # given
        self.client.force_login(self.user_1002)

        timezone_info = TimezoneData.objects.create(
            timezone_name="Europe/Berlin", utc_offset="+0100", panel_id="europe-berlin"
        )

        Timezones.objects.create(
            panel_name="Europe/Berlin", is_enabled=1, timezone=timezone_info
        )

        # when
        res = self.client.get(reverse("timezones:index"))

        # then
        self.assertQuerysetEqual(
            res.context["timezones"], Timezones.objects.all(), transform=lambda x: x
        )



# Generated by Django 3.1.1 on 2020-10-03 17:48

# Django
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("timezones", "0002_timezonedata_timezones"),
    ]

    operations = [
        migrations.AlterField(
            model_name="timezonedata",
            name="utc_offset",
            field=models.CharField(
                help_text="UTC offset of the timezone", max_length=255
            ),
        ),
    ]
